import json
import requests
import threading
import time
from pathlib import Path
from PIL import Image, ImageDraw, ImageFont
import pystray
from pystray import MenuItem as item

from src.py.utils.constants import LOG_PATH, PROJECT_ROOT, AUTH_TOKEN

# 请求 URL 和 headers
URL = "https://gaccode.com/api/credits/balance"
HEADERS = {
    "Accept": "*/*",
    "Accept-Encoding": "gzip, deflate, br, zstd",
    "Accept-Language": "zh",
    "Authorization": AUTH_TOKEN,
    "Content-Type": "application/json",
    "Referer": "https://gaccode.com/credits",
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/136.0.0.0 Safari/537.36 Edg/136.0.0.0"
}

# 全局变量存储余额数据
balance_data = {
    "balance": 0,
    "creditCap": 0,
    "refillRate": 0,
    "lastRefill": ""
}

def create_image(balance=0, credit_cap=0):
    """创建显示余额的图像 - 类似任务管理器的竖向水槽"""
    # 创建一个 32x64 的透明图像 (竖向长方形)
    width = 32
    height = 64
    image = Image.new('RGBA', (width, height), color=(0, 0, 0, 0))
    draw = ImageDraw.Draw(image)
    
    # 计算余额百分比
    percentage = min(100, int(balance / credit_cap * 100)) if credit_cap > 0 else 0
    
    # 绘制长方形边框 (2像素宽)
    border_color = (180, 180, 180, 255)  # 灰色边框
    draw.rectangle([(0, 0), (width-1, height-1)], fill=None, outline=border_color, width=2)
    
    # 内部填充区域
    inner_margin = 4
    inner_width = width - (inner_margin * 2)
    inner_height = height - (inner_margin * 2)
    
    # 绘制背景 (深色)
    background_color = (50, 50, 50, 200)  # 深灰色背景
    draw.rectangle([(inner_margin, inner_margin), 
                   (width-inner_margin-1, height-inner_margin-1)], 
                   fill=background_color)
    
    # 计算水槽高度 (百分比反向，从底部往上填)
    water_height = int((percentage / 100) * inner_height)
    water_top = height - inner_margin - water_height
    
    # 绘制蓝色水槽
    # 根据百分比调整颜色 - 低时为红色，中等为黄色，高时为蓝色
    if percentage > 70:
        water_color = (0, 120, 255, 220)  # 蓝色
    elif percentage > 30:
        water_color = (255, 200, 0, 220)   # 黄色
    else:
        water_color = (255, 60, 60, 220)   # 红色
        
    if water_height > 0:
        draw.rectangle([
            (inner_margin, water_top),
            (width-inner_margin-1, height-inner_margin-1)
        ], fill=water_color)
    
    return image

def fetch_balance():
    """获取GACCode的积分余额"""
    global balance_data
    try:
        response = requests.get(URL, headers=HEADERS, timeout=10)
        if response.status_code == 200:
            # 更新全局数据
            balance_data = json.loads(response.text)
            return balance_data
        else:
            print(f"请求失败，状态码: {response.status_code}")
    except Exception as e:
        print(f"发生异常: {str(e)}")
    return None

def update_icon(icon):
    """更新系统托盘图标"""
    global balance_data
    while True:
        new_data = fetch_balance()
        if new_data:
            # 更新图标
            icon.icon = create_image(new_data["balance"], new_data["creditCap"])
            # 更新提示文本
            icon.title = f"GACCode 积分: {new_data['balance']}/{new_data['creditCap']} ({int(new_data['balance']/new_data['creditCap']*100)}%)"
        
        # 每分钟更新一次
        time.sleep(60)

def read_last_log():
    """读取最后一行日志来获取余额信息"""
    try:
        with open(LOG_PATH, "r", encoding="utf-8") as f:
            lines = f.readlines()
            if lines:
                last_line = lines[-1].strip()
                # 尝试解析JSON部分
                try:
                    json_start = last_line.find('{"balance":')
                    json_end = last_line.rfind('}') + 1
                    if json_start != -1 and json_end != -1:
                        json_str = last_line[json_start:json_end]
                        data = json.loads(json_str)
                        return data
                except:
                    pass
    except Exception as e:
        print(f"读取日志失败: {str(e)}")
    return None

def exit_action(icon):
    icon.stop()

def setup_tray():
    # 初始化余额数据（尝试从日志中读取）
    global balance_data
    log_data = read_last_log()
    if log_data:
        balance_data = log_data
    
    # 创建初始图标
    image = create_image(balance_data["balance"], balance_data["creditCap"])
    
    # 创建系统托盘图标
    icon = pystray.Icon(
        "gaccode_balance",
        image,
        f"GACCode 积分: {balance_data['balance']}/{balance_data['creditCap']}",
        menu=pystray.Menu(
            item('退出', exit_action)
        )
    )
    
    # 启动更新线程
    threading.Thread(target=update_icon, args=(icon,), daemon=True).start()
    
    # 运行托盘图标
    icon.run()

if __name__ == "__main__":
    setup_tray() 


import logging
import sys
from datetime import datetime
from pathlib import Path


class LogUtil:
    """
    日志工具类，提供同时输出到控制台和文件的功能
    日志文件按天生成，存储在项目根目录的logs目录下
    """
    
    _loggers = {}
    
    @classmethod
    def get_logger(cls, name='getDialog', level=logging.INFO):
        """
        获取日志记录器
        
        Args:
            name: 日志记录器名称
            level: 日志级别
            
        Returns:
            logging.Logger: 配置好的日志记录器
        """
        if name in cls._loggers:
            return cls._loggers[name]
        
        logger = logging.getLogger(name)
        logger.setLevel(level)
        
        # 清除已有的处理器
        logger.handlers.clear()
        
        # 创建格式化器
        formatter = logging.Formatter(
            '%(asctime)s - %(name)s - %(levelname)s - %(message)s',
            datefmt='%Y-%m-%d %H:%M:%S'
        )
        
        # 控制台处理器
        console_handler = logging.StreamHandler(sys.stdout)
        console_handler.setLevel(level)
        console_handler.setFormatter(formatter)
        logger.addHandler(console_handler)
        
        # 文件处理器
        file_handler = cls._create_file_handler(level, formatter)
        logger.addHandler(file_handler)
        
        # 避免重复日志
        logger.propagate = False
        
        cls._loggers[name] = logger
        return logger
    
    @classmethod
    def _create_file_handler(cls, level, formatter):
        """
        创建文件处理器
        
        Args:
            level: 日志级别
            formatter: 格式化器
            
        Returns:
            logging.FileHandler: 文件处理器
        """
        # 获取项目根目录
        project_root = Path(__file__).parent.parent.parent.parent
        logs_dir = project_root / 'logs'
        
        # 确保日志目录存在
        logs_dir.mkdir(exist_ok=True)
        
        # 生成日志文件名（按天）
        today = datetime.now().strftime('%Y-%m-%d')
        log_file = logs_dir / f'getDialog_{today}.log'
        
        # 创建文件处理器
        file_handler = logging.FileHandler(log_file, encoding='utf-8')
        file_handler.setLevel(level)
        file_handler.setFormatter(formatter)
        
        return file_handler
    
    @classmethod
    def info(cls, message, logger_name='getDialog'):
        """记录INFO级别日志"""
        logger = cls.get_logger(logger_name)
        logger.info(message)
    
    @classmethod
    def debug(cls, message, logger_name='getDialog'):
        """记录DEBUG级别日志"""
        logger = cls.get_logger(logger_name)
        logger.debug(message)
    
    @classmethod
    def warning(cls, message, logger_name='getDialog'):
        """记录WARNING级别日志"""
        logger = cls.get_logger(logger_name)
        logger.warning(message)
    
    @classmethod
    def error(cls, message, logger_name='getDialog'):
        """记录ERROR级别日志"""
        logger = cls.get_logger(logger_name)
        logger.error(message)
    
    @classmethod
    def critical(cls, message, logger_name='getDialog'):
        """记录CRITICAL级别日志"""
        logger = cls.get_logger(logger_name)
        logger.critical(message)


# 提供默认实例
default_logger = LogUtil.get_logger()


def log_info(message):
    """快捷方法：记录INFO级别日志"""
    default_logger.info(message)


def log_debug(message):
    """快捷方法：记录DEBUG级别日志"""
    default_logger.debug(message)


def log_warning(message):
    """快捷方法：记录WARNING级别日志"""
    default_logger.warning(message)


def log_error(message):
    """快捷方法：记录ERROR级别日志"""
    default_logger.error(message)


def log_critical(message):
    """快捷方法：记录CRITICAL级别日志"""
    default_logger.critical(message)


import requests
import schedule
import time
import datetime

from src.py.utils.constants import LOG_PATH, AUTH_TOKEN

# 请求 URL 和 headers
URL = "https://gaccode.com/api/credits/balance"
HEADERS = {
    "Accept": "*/*",
    "Accept-Encoding": "gzip, deflate, br, zstd",
    "Accept-Language": "zh",
    "Authorization": AUTH_TOKEN,
    "Content-Type": "application/json",
    "Referer": "https://gaccode.com/credits",
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/136.0.0.0 Safari/537.36 Edg/136.0.0.0"
}


def write_log(message):
    with open(LOG_PATH, "a", encoding="utf-8") as f:
        f.write(message + "\n")

def fetch_balance():
    now = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    try:
        response = requests.get(URL, headers=HEADERS, timeout=10)
        if response.status_code == 200:
            log_message = f"[{now}] ✅ 200 OK - Response: {response.text}"
        elif response.status_code == 401:
            log_message = f"[{now}] ❌ 401 Unauthorized - Token may be expired"
        else:
            log_message = f"[{now}] ⚠️ {response.status_code} - Response: {response.text}"
    except Exception as e:
        log_message = f"[{now}] ❗ Exception occurred: {str(e)}"

    print(log_message)
    write_log(log_message)

def main():
    # 每 1 分钟执行一次
    minute = 1
    schedule.every(minute).minutes.do(fetch_balance)

    # 启动时立即执行一次
    fetch_balance()

    print(f"⏳ 定时任务已启动（每{minute}分钟执行一次），日志写入 {LOG_PATH}")
    while True:
        schedule.run_pending()
        time.sleep(10)

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
项目常量定义
"""

import os
import configparser
from pathlib import Path

# 项目根目录 - 从当前文件向上4级
PROJECT_ROOT = Path(__file__).parent.parent.parent.parent
print("PROJECT_ROOT", PROJECT_ROOT)

# 日志路径
LOG_PATH = PROJECT_ROOT / "logs" / "fetch_gaccode_balance.log"
os.makedirs(PROJECT_ROOT / "logs", exist_ok=True)

# 配置文件路径
CONFIG_PATH = PROJECT_ROOT / "config.ini"

def get_token_from_config():
    """从配置文件获取token"""
    if not CONFIG_PATH.exists():
        # 创建默认配置文件
        config = configparser.ConfigParser()
        config['DEFAULT'] = {'token': ''}
        with open(CONFIG_PATH, 'w') as configfile:
            config.write(configfile)
        return ""
    
    # 尝试读取老格式配置（不带节的配置文件）
    try:
        with open(CONFIG_PATH, 'r') as f:
            content = f.read().strip()
            if content.startswith('token='):
                return content.split('=', 1)[1]
    except Exception:
        pass
    
    # 读取标准ini格式
    try:
        config = configparser.ConfigParser()
        config.read(CONFIG_PATH)
        return config.get('DEFAULT', 'token', fallback='')
    except Exception:
        return ""

# 从配置获取token
AUTH_TOKEN = get_token_from_config()



from setuptools import setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="gaccode-monitor",
    version="0.1.0",
    author="GACCode Monitor Team",
    author_email="yourname@example.com",
    description="A system tray application to monitor GACCode credits balance",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/yourusername/watchGacCredits",
    packages=["py", "py.utils"],
    package_dir={"": "src"},
    package_data={
        "py": ["*.py"],
        "py.utils": ["*.py"],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Microsoft :: Windows",
    ],
    python_requires=">=3.6",
    install_requires=[
        "requests",
        "schedule",
        "pystray",
        "pillow",
    ],
    entry_points={
        "console_scripts": [
            "gaccode-monitor=py.gaccode_tray_icon:setup_tray",
            "gaccode-logger=py.fetch_gaccode_balance:main",
        ],
    },
) 


from ast import main
import time
import threading
import json
from utils.constants import CONFIG_FILE
from utils.log_util import default_logger as logger


class UniqueIDGenerator:
    """
    单例模式的唯一ID生成器
    基于时间戳和序列号生成6位唯一ID
    """

    _instance = None
    _lock = threading.Lock()

    def __new__(cls):
        """
        单例模式实现，确保只创建一个UniqueIDGenerator实例

        Returns:
            UniqueIDGenerator: 唯一的类实例
        """
        with cls._lock:
            if not cls._instance:
                cls._instance = super().__new__(cls)
                cls._instance.initial_time = int(time.time())
                cls._instance.sequence = 0
                cls._instance.load_config()
            return cls._instance

    def load_config(self):
        """
        从配置文件加载初始时间戳和序列号

        Returns:
            None
        """
        if CONFIG_FILE.exists():
            with open(CONFIG_FILE, "r") as f:
                self.config = json.load(f)
                self.initial_time = self.config["id"]["current_value"]
                self.sequence = self.config["id"]["current_value"]

    def save_config(self):
        """
        保存当前配置到配置文件

        Returns:
            None
        """
        self.config["id"]["current_value"] = self.sequence
        with open(CONFIG_FILE, "w") as f:
            json.dump(self.config, f, indent=2)
            logger.info(f"配置已保存到: {CONFIG_FILE},id:{self.sequence}") 

    def generate_id(self):
        """
        生成唯一ID
        Returns:
            str: 6位数字id 100000
        """

        with self._lock:
            self.sequence += 1
            self.save_config()
            logger.info(f"当前ID: {self.sequence}")
            return f"{self.sequence % 1000000:06d}"


def generate_ids(thread_id, count):
    """
    多线程环境下生成多个唯一ID

    Args:
        thread_id (int): 线程ID，用于标识不同线程
        count (int): 需要生成的ID数量
    """
    generator = UniqueIDGenerator()
    results = []
    for _ in range(count):
        results.append(generator.generate_id())
        time.sleep(0.001)
    print(f"线程 {thread_id} 生成的ID: {', '.join(results)}")


def test_thread_generate():
    """
    测试多线程生成ID
    """
    threads = []
    for i in range(10):  # 创建10个线程
        thread = threading.Thread(
            target=generate_ids, args=(i, 10)  # 每个线程生成10个ID
        )
        threads.append(thread)
        thread.start()

    # 等待所有线程完成
    for thread in threads:
        thread.join()


def test_single():
    """
    测试单线程生成ID
    """
    generator = UniqueIDGenerator()
    for i in range(10):
        print(generator.generate_id())


if __name__ == "__main__":
    test_thread_generate()
    # test_single()
